from .species import Species
from ..quantities import *


class C5H10(Species):
    def __init__(self, T=None):
        self.mol_weight = 70.135
        self.c_atoms = 5
        self.h_atoms = 10
        super().__init__(T)
